
#include "local.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "son.h"


/******************************************************************
	Expands out istr.

	Assumes that istr contains a string which includes
	environment variables in the form of $VAR or ${VAR}
	This routing will expands out the environment variables
	and return the expanded string.  The expanded string is an
	allocated string using new().  It should be deallocated with
	sFree().

	Examples or good istr strings:
		$SONNET_DIR/stiplist/stiplist
		${HOME}/file
		$HOME/$NAME$MACH
		$HOME/${MACH}aaa

* Copyright (c) 1989 Sonnet Software, Inc., All rights reserved. *
******************************************************************/

LOCAL_FN char *
expand_variable(
#if NeedFunctionPrototypes
	char *var
#endif
);

LOCAL_FN void
get_variable(
#if NeedFunctionPrototypes
	char *str,
	int *num,
	char **var
#endif
);

char *
expstr(istr)
char *istr;
{
	char *ostr;
	int len;
	char *token;
	int olen;
	char *var;
	extern char *getenv();
	int inu;
	int onu;

	len = strlen(istr);
	olen = len + 1;
	ostr = (char *)new(sizeof(char)*olen);
        ostr[0] = '\0';
	inu = 0;
	onu = 0;
	
	while ( istr[inu] != '\0' ) {
		if ( istr[inu]  == '$' ) {
			/* found a variable */
			get_variable( istr,&inu,&var);
			ostr[onu] = '\0';
			olen = olen - strlen(var)-1;
			token = expand_variable(var);
			if ( token != NULL ) {
				olen += strlen(token);
				ostr = rnew(ostr,sizeof(char) * olen );
				strcat(ostr,token);
				onu += strlen(token);
			}
			sFree(var);
		} else {
			ostr[onu++] = istr[inu++];
		}
	}
	ostr[onu] = '\0';
	return (ostr);
}

LOCAL_FN void
get_variable(str,num,var)
char *str;
int *num;
char **var;
{
	int i;
	int n;
	int n_orig;
	
	n = *num;

	/* skip over '$' */
	n += 1;		

	if ( str[n] == '{' ) ++n;
	i=0;

	n_orig = n;
	
	/* 
	** Variable characters may be alpha-numeric or underscore
	** Look for next non-variable character.
	*/
	while (  str[n] != '\0' ) {
		if ( ! isalnum(str[n]) && str[n] != '_' ) {
			break;
		}
		++i;
		++n;
	}
	if ( n == n_orig ) {
		++i;
		++n;
	}
	*var = (char *)new ( sizeof(char)*i + 1 );
	strncpy(*var,str+n_orig,i);
	(*var)[i] = '\0';
	if ( str[n] == '}' ) ++n;
	*num = n;
}


/******************************************************************
	Expand out environment variable.  If the variable is "$"
	the sub in the process id.
******************************************************************/
LOCAL_FN char *
expand_variable(var)
char *var;
{
	int pid;
	static char str[20];

	if ( strcmp(var,"$") == 0 ) {
		sprintf(str,"%d",getpid());
		return (str);
	}

	return ( getenv(var) );
}
